import graphene

from ..logs.events import GraphenePipelineRunEvent
from ..util import non_null_list
from .pipeline import GraphenePipelineRun


class GraphenePipelineRunLogsSubscriptionSuccess(graphene.ObjectType):
    run = graphene.NonNull(GraphenePipelineRun)
    messages = non_null_list(GraphenePipelineRunEvent)

    class Meta:
        name = "PipelineRunLogsSubscriptionSuccess"


class GraphenePipelineRunLogsSubscriptionFailure(graphene.ObjectType):
    message = graphene.NonNull(graphene.String)
    missingRunId = graphene.Field(graphene.String)

    class Meta:
        name = "PipelineRunLogsSubscriptionFailure"


class GraphenePipelineRunLogsSubscriptionPayload(graphene.Union):
    class Meta:
        types = (
            GraphenePipelineRunLogsSubscriptionSuccess,
            GraphenePipelineRunLogsSubscriptionFailure,
        )
        name = "PipelineRunLogsSubscriptionPayload"



from dagster_graphql.test.utils import execute_dagster_graphql, infer_pipeline_selector

RESOURCE_QUERY = """
query ResourceQuery($selector: PipelineSelector!) {
  pipelineOrError(params: $selector) {
    __typename
    ... on Pipeline {
      modes {
        name
        resources {
          name
          description
          configField {
            configType {
              key
              ... on CompositeConfigType {
                fields {
                  name
                  configType {
                    key
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}
"""

REQUIRED_RESOURCE_QUERY = """
query RequiredResourceQuery($selector: PipelineSelector!) {
  pipelineOrError(params: $selector) {
    ... on Pipeline {
      name
      solids {
        definition {
            requiredResources {
              resourceKey
            }
        }
      }
    }
  }
}
"""


def test_mode_fetch_resources(graphql_context, snapshot):
    selector = infer_pipeline_selector(graphql_context, "multi_mode_with_resources")
    result = execute_dagster_graphql(
        graphql_context,
        RESOURCE_QUERY,
        {"selector": selector},
    )

    assert not result.errors
    assert result.data
    assert result.data["pipelineOrError"]
    assert result.data["pipelineOrError"]["modes"]
    for mode_data in result.data["pipelineOrError"]["modes"]:
        assert mode_data["resources"]

    snapshot.assert_match(result.data)


# Warning: If _compute_fields_hash changes, verify that the result.data has the same shape/keys/values
# as the existing snapshot and then run update snapshot
def test_required_resources(graphql_context, snapshot):
    selector = infer_pipeline_selector(graphql_context, "required_resource_pipeline")
    result = execute_dagster_graphql(
        graphql_context,
        REQUIRED_RESOURCE_QUERY,
        {"selector": selector},
    )

    assert not result.errors
    assert result.data
    assert result.data["pipelineOrError"]["solids"]
    [solid] = result.data["pipelineOrError"]["solids"]
    assert solid
    assert solid["definition"]["requiredResources"]
    assert solid["definition"]["requiredResources"] == [{"resourceKey": "R1"}]

    snapshot.assert_match(result.data)



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots['test_query_all_solids 1'] = {
    'repositoryOrError': {
        'usedSolids': [
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'a_solid_with_multilayered_config'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'more_complicated_nested_config'
                        },
                        'solidHandle': {
                            'handleID': 'a_solid_with_multilayered_config'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'a_solid_with_three_field_config'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'more_complicated_config'
                        },
                        'solidHandle': {
                            'handleID': 'a_solid_with_three_field_config'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'add_four'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'composites_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'add_four'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'add_one'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'composites_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'add_four.adder_1.adder_1'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'composites_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'add_four.adder_1.adder_2'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'composites_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'add_four.adder_2.adder_1'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'composites_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'add_four.adder_2.adder_2'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'add_two'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'composites_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'add_four.adder_1'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'composites_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'add_four.adder_2'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'after_failure'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'chained_failure_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'after_failure'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'alp_a'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'asset_lineage_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'alp_a'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'alp_b'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'asset_lineage_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'alp_b'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'always_succeed'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'chained_failure_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'always_succeed'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'apply_to_three'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'multi_mode_with_resources'
                        },
                        'solidHandle': {
                            'handleID': 'apply_to_three'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'can_fail'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'retry_multi_output_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'can_fail'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'conditionally_fail'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'chained_failure_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'conditionally_fail'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'df_expectations_solid'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'csv_hello_world_with_expectations'
                        },
                        'solidHandle': {
                            'handleID': 'df_expectations_solid'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'div_four'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'composites_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'div_four'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'div_two'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'composites_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'div_four.div_1'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'composites_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'div_four.div_2'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'emit'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'dynamic_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'emit'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'emit_failed_expectation'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'pipeline_with_expectations'
                        },
                        'solidHandle': {
                            'handleID': 'emit_failed_expectation'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'emit_successful_expectation'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'pipeline_with_expectations'
                        },
                        'solidHandle': {
                            'handleID': 'emit_successful_expectation'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'emit_successful_expectation_no_metadata'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'pipeline_with_expectations'
                        },
                        'solidHandle': {
                            'handleID': 'emit_successful_expectation_no_metadata'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'emit_ten'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'dynamic_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'emit_ten'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'fail'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'eventually_successful'
                        },
                        'solidHandle': {
                            'handleID': 'fail'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'eventually_successful'
                        },
                        'solidHandle': {
                            'handleID': 'fail_2'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'eventually_successful'
                        },
                        'solidHandle': {
                            'handleID': 'fail_3'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'fail_subset'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'pipeline_with_invalid_definition_error'
                        },
                        'solidHandle': {
                            'handleID': 'fail_subset'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'get_input_one'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'retry_multi_input_early_terminate_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'get_input_one'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'get_input_two'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'retry_multi_input_early_terminate_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'get_input_two'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'hard_fail_or_0'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'hard_failer'
                        },
                        'solidHandle': {
                            'handleID': 'hard_fail_or_0'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'increment'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'hard_failer'
                        },
                        'solidHandle': {
                            'handleID': 'increment'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'loop'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'infinite_loop_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'loop'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'materialize'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'materialization_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'materialize'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'multi'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'retry_multi_output_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'multi'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'multiply_by_two'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'dynamic_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'double_total'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'dynamic_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'multiply_by_two'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'multiply_inputs'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'dynamic_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'multiply_inputs'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'no_output'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'retry_multi_output_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'child_multi_skip'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'retry_multi_output_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'child_skip'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'retry_multi_output_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'grandchild_fail'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'noop_solid'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'asset_lineage_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'noop_solid'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'more_complicated_config'
                        },
                        'solidHandle': {
                            'handleID': 'noop_solid'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'noop_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'noop_solid'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'partitioned_asset_lineage_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'noop_solid'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'one'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'pipeline_with_invalid_definition_error'
                        },
                        'solidHandle': {
                            'handleID': 'one'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'palp_a'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'partitioned_asset_lineage_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'palp_a'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'palp_b'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'partitioned_asset_lineage_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'palp_b'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'passthrough'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'retry_multi_output_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'child_fail'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'reset'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'eventually_successful'
                        },
                        'solidHandle': {
                            'handleID': 'reset'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'return_any'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'scalar_output_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'return_any'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'return_bool'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'scalar_output_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'return_bool'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'return_foo'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'no_config_chain_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'return_foo'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'return_hello'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'no_config_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'return_hello'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'return_hello_world'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'no_config_chain_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'return_hello_world'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'return_int'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'scalar_output_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'return_int'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'return_one'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'retry_multi_input_early_terminate_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'return_one'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'return_six'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'multi_mode_with_loggers'
                        },
                        'solidHandle': {
                            'handleID': 'return_six'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'return_str'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'scalar_output_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'return_str'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'simple_solid'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'tagged_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'simple_solid'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'solid_asset_a'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'multi_asset_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'solid_asset_a'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'single_asset_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'solid_asset_a'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'solid_asset_b'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'multi_asset_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'solid_asset_b'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'solid_partitioned_asset'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'partitioned_asset_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'solid_partitioned_asset'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'solid_that_gets_tags'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'hello_world_with_tags'
                        },
                        'solidHandle': {
                            'handleID': 'solid_that_gets_tags'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'solid_with_list'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'pipeline_with_list'
                        },
                        'solidHandle': {
                            'handleID': 'solid_with_list'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'solid_with_required_resource'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'required_resource_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'solid_with_required_resource'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'spawn'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'eventually_successful'
                        },
                        'solidHandle': {
                            'handleID': 'spawn'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'spew'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'spew_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'spew'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'start'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'retry_resource_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'start'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'sum_inputs'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'retry_multi_input_early_terminate_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'sum_inputs'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'sum_numbers'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'dynamic_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'sum_numbers'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'sum_solid'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'csv_hello_world'
                        },
                        'solidHandle': {
                            'handleID': 'sum_solid'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'csv_hello_world_df_input'
                        },
                        'solidHandle': {
                            'handleID': 'sum_solid'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'csv_hello_world_two'
                        },
                        'solidHandle': {
                            'handleID': 'sum_solid'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'csv_hello_world_with_expectations'
                        },
                        'solidHandle': {
                            'handleID': 'sum_solid'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'sum_sq_solid'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'csv_hello_world'
                        },
                        'solidHandle': {
                            'handleID': 'sum_sq_solid'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'csv_hello_world_df_input'
                        },
                        'solidHandle': {
                            'handleID': 'sum_sq_solid'
                        }
                    },
                    {
                        'pipeline': {
                            'name': 'csv_hello_world_with_expectations'
                        },
                        'solidHandle': {
                            'handleID': 'sum_sq_solid'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'tag_asset_solid'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'asset_tag_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'tag_asset_solid'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'takes_an_enum'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'pipeline_with_enum_config'
                        },
                        'solidHandle': {
                            'handleID': 'takes_an_enum'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'throw_a_thing'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'naughty_programmer_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'throw_a_thing'
                        }
                    }
                ]
            },
            {
                '__typename': 'UsedSolid',
                'definition': {
                    'name': 'will_fail'
                },
                'invocations': [
                    {
                        'pipeline': {
                            'name': 'retry_resource_pipeline'
                        },
                        'solidHandle': {
                            'handleID': 'will_fail'
                        }
                    }
                ]
            }
        ]
    }
}



import graphene

from .config_types import (
    GrapheneCompositeConfigType,
    GrapheneEnumConfigType,
    GrapheneRegularConfigType,
)
from .errors import GraphenePipelineNotFoundError, GraphenePythonError
from .pipelines.pipeline_errors import GrapheneConfigTypeNotFoundError


class GrapheneConfigTypeOrError(graphene.Union):
    class Meta:
        types = (
            GrapheneEnumConfigType,
            GrapheneCompositeConfigType,
            GrapheneRegularConfigType,
            GraphenePipelineNotFoundError,
            GrapheneConfigTypeNotFoundError,
            GraphenePythonError,
        )
        name = "ConfigTypeOrError"



import graphene


class GraphenePipelineRunStatus(graphene.Enum):
    QUEUED = "QUEUED"
    NOT_STARTED = "NOT_STARTED"
    MANAGED = "MANAGED"
    STARTING = "STARTING"
    STARTED = "STARTED"
    SUCCESS = "SUCCESS"
    FAILURE = "FAILURE"
    CANCELING = "CANCELING"
    CANCELED = "CANCELED"

    class Meta:
        name = "PipelineRunStatus"



import os
import time

from dagster import execute_pipeline
from dagster.core.definitions.reconstructable import ReconstructableRepository
from dagster.core.storage.pipeline_run import PipelineRunStatus
from dagster.grpc.types import CancelExecutionRequest
from dagster.utils import file_relative_path, safe_tempfile_path
from dagster_graphql.client.query import LAUNCH_PIPELINE_EXECUTION_MUTATION
from dagster_graphql.test.utils import execute_dagster_graphql, infer_pipeline_selector

from .graphql_context_test_suite import GraphQLContextVariant, make_graphql_context_test_suite

RUN_CANCELLATION_QUERY = """
mutation($runId: String!, $terminatePolicy: TerminatePipelinePolicy) {
  terminatePipelineExecution(runId: $runId, terminatePolicy:$terminatePolicy){
    __typename
    ... on TerminatePipelineExecutionSuccess{
      run {
        runId
      }
    }
    ... on TerminatePipelineExecutionFailure {
      run {
        runId
      }
      message
    }
    ... on PipelineRunNotFoundError {
      runId
    }
    ... on PythonError {
      message
      stack
    }
  }
}
"""


class TestQueuedRunTermination(
    make_graphql_context_test_suite(
        context_variants=[
            GraphQLContextVariant.sqlite_with_queued_run_coordinator_managed_grpc_env()
        ]
    )
):
    def test_cancel_queued_run(self, graphql_context):
        selector = infer_pipeline_selector(graphql_context, "infinite_loop_pipeline")
        with safe_tempfile_path() as path:
            result = execute_dagster_graphql(
                graphql_context,
                LAUNCH_PIPELINE_EXECUTION_MUTATION,
                variables={
                    "executionParams": {
                        "selector": selector,
                        "mode": "default",
                        "runConfigData": {"solids": {"loop": {"config": {"file": path}}}},
                    }
                },
            )

            assert not result.errors
            assert result.data

            # just test existence
            assert (
                result.data["launchPipelineExecution"]["__typename"] == "LaunchPipelineRunSuccess"
            )
            run_id = result.data["launchPipelineExecution"]["run"]["runId"]

            assert graphql_context.instance.get_run_by_id(run_id).status == PipelineRunStatus.QUEUED

            result = execute_dagster_graphql(
                graphql_context, RUN_CANCELLATION_QUERY, variables={"runId": run_id}
            )
            assert (
                result.data["terminatePipelineExecution"]["__typename"]
                == "TerminatePipelineExecutionSuccess"
            )

    def test_force_cancel_queued_run(self, graphql_context):
        selector = infer_pipeline_selector(graphql_context, "infinite_loop_pipeline")
        with safe_tempfile_path() as path:
            result = execute_dagster_graphql(
                graphql_context,
                LAUNCH_PIPELINE_EXECUTION_MUTATION,
                variables={
                    "executionParams": {
                        "selector": selector,
                        "mode": "default",
                        "runConfigData": {"solids": {"loop": {"config": {"file": path}}}},
                    }
                },
            )

            assert not result.errors
            assert result.data

            # just test existence
            assert (
                result.data["launchPipelineExecution"]["__typename"] == "LaunchPipelineRunSuccess"
            )
            run_id = result.data["launchPipelineExecution"]["run"]["runId"]

            assert graphql_context.instance.get_run_by_id(run_id).status == PipelineRunStatus.QUEUED

            result = execute_dagster_graphql(
                graphql_context,
                RUN_CANCELLATION_QUERY,
                variables={"runId": run_id, "terminatePolicy": "MARK_AS_CANCELED_IMMEDIATELY"},
            )
            assert (
                result.data["terminatePipelineExecution"]["__typename"]
                == "TerminatePipelineExecutionSuccess"
            )


class TestRunVariantTermination(
    make_graphql_context_test_suite(
        context_variants=[GraphQLContextVariant.sqlite_with_default_run_launcher_managed_grpc_env()]
    )
):
    def test_basic_termination(self, graphql_context):
        selector = infer_pipeline_selector(graphql_context, "infinite_loop_pipeline")
        with safe_tempfile_path() as path:
            result = execute_dagster_graphql(
                graphql_context,
                LAUNCH_PIPELINE_EXECUTION_MUTATION,
                variables={
                    "executionParams": {
                        "selector": selector,
                        "mode": "default",
                        "runConfigData": {"solids": {"loop": {"config": {"file": path}}}},
                    }
                },
            )

            assert not result.errors
            assert result.data

            # just test existence
            assert (
                result.data["launchPipelineExecution"]["__typename"] == "LaunchPipelineRunSuccess"
            )
            run_id = result.data["launchPipelineExecution"]["run"]["runId"]

            assert run_id

            # ensure the execution has happened
            while not os.path.exists(path):
                time.sleep(0.1)

            result = execute_dagster_graphql(
                graphql_context, RUN_CANCELLATION_QUERY, variables={"runId": run_id}
            )
            assert (
                result.data["terminatePipelineExecution"]["__typename"]
                == "TerminatePipelineExecutionSuccess"
            )

    def test_run_not_found(self, graphql_context):
        result = execute_dagster_graphql(
            graphql_context, RUN_CANCELLATION_QUERY, variables={"runId": "nope"}
        )
        assert result.data["terminatePipelineExecution"]["__typename"] == "PipelineRunNotFoundError"

    def test_terminate_failed(self, graphql_context):
        selector = infer_pipeline_selector(graphql_context, "infinite_loop_pipeline")
        with safe_tempfile_path() as path:
            old_terminate = graphql_context.instance.run_launcher.terminate
            graphql_context.instance.run_launcher.terminate = lambda _run_id: False
            result = execute_dagster_graphql(
                graphql_context,
                LAUNCH_PIPELINE_EXECUTION_MUTATION,
                variables={
                    "executionParams": {
                        "selector": selector,
                        "mode": "default",
                        "runConfigData": {"solids": {"loop": {"config": {"file": path}}}},
                    }
                },
            )

            assert not result.errors
            assert result.data

            # just test existence
            assert (
                result.data["launchPipelineExecution"]["__typename"] == "LaunchPipelineRunSuccess"
            )
            run_id = result.data["launchPipelineExecution"]["run"]["runId"]
            # ensure the execution has happened
            while not os.path.exists(path):
                time.sleep(0.1)

            result = execute_dagster_graphql(
                graphql_context, RUN_CANCELLATION_QUERY, variables={"runId": run_id}
            )
            assert (
                result.data["terminatePipelineExecution"]["__typename"]
                == "TerminatePipelineExecutionFailure"
            )
            assert result.data["terminatePipelineExecution"]["message"].startswith(
                "Unable to terminate run"
            )

            result = execute_dagster_graphql(
                graphql_context,
                RUN_CANCELLATION_QUERY,
                variables={"runId": run_id, "terminatePolicy": "MARK_AS_CANCELED_IMMEDIATELY"},
            )

            assert (
                result.data["terminatePipelineExecution"]["__typename"]
                == "TerminatePipelineExecutionSuccess"
            )

            assert result.data["terminatePipelineExecution"]["run"]["runId"] == run_id

            graphql_context.instance.run_launcher.terminate = old_terminate

            # Clean up the run process on the gRPC server
            repository_location = graphql_context.repository_locations[0]
            repository_location.client.cancel_execution(CancelExecutionRequest(run_id=run_id))

    def test_run_finished(self, graphql_context):
        instance = graphql_context.instance

        pipeline = ReconstructableRepository.for_file(
            file_relative_path(__file__, "setup.py"),
            "test_repo",
        ).get_reconstructable_pipeline("noop_pipeline")

        pipeline_result = execute_pipeline(pipeline, instance=instance)
        assert pipeline_result.success
        assert pipeline_result.run_id

        time.sleep(0.05)  # guarantee execution finish

        result = execute_dagster_graphql(
            graphql_context, RUN_CANCELLATION_QUERY, variables={"runId": pipeline_result.run_id}
        )

        assert (
            result.data["terminatePipelineExecution"]["__typename"]
            == "TerminatePipelineExecutionFailure"
        )
        assert (
            "could not be terminated due to having status SUCCESS."
            in result.data["terminatePipelineExecution"]["message"]
        )

        # Still fails even if you change the terminate policy to fail immediately
        result = execute_dagster_graphql(
            graphql_context,
            RUN_CANCELLATION_QUERY,
            variables={
                "runId": pipeline_result.run_id,
                "terminatePolicy": "MARK_AS_CANCELED_IMMEDIATELY",
            },
        )

        assert (
            result.data["terminatePipelineExecution"]["__typename"]
            == "TerminatePipelineExecutionFailure"
        )
        assert (
            "could not be terminated due to having status SUCCESS."
            in result.data["terminatePipelineExecution"]["message"]
        )



from collections import OrderedDict

from dagster_graphql.client.query import LAUNCH_PARTITION_BACKFILL_MUTATION
from dagster_graphql.test.utils import (
    execute_dagster_graphql,
    execute_dagster_graphql_and_finish_runs,
    infer_repository_selector,
)

from .graphql_context_test_suite import (
    ExecutingGraphQLContextTestMatrix,
    ReadonlyGraphQLContextTestMatrix,
)

GET_PARTITION_SETS_FOR_PIPELINE_QUERY = """
    query PartitionSetsQuery($repositorySelector: RepositorySelector!, $pipelineName: String!) {
        partitionSetsOrError(repositorySelector: $repositorySelector, pipelineName: $pipelineName) {
            __typename
            ...on PartitionSets {
                results {
                    name
                    pipelineName
                    solidSelection
                    mode
                }
            }
            ... on PythonError {
                message
                stack
            }
            ...on PipelineNotFoundError {
                message
            }
        }
    }
"""

GET_PARTITION_SET_QUERY = """
    query PartitionSetQuery($repositorySelector: RepositorySelector!, $partitionSetName: String!) {
        partitionSetOrError(repositorySelector: $repositorySelector, partitionSetName: $partitionSetName) {
            __typename
            ... on PythonError {
                message
                stack
            }
            ...on PartitionSet {
                name
                pipelineName
                solidSelection
                mode
                partitionsOrError {
                    ... on Partitions {
                        results {
                            name
                            tagsOrError {
                                __typename
                            }
                            runConfigOrError {
                                ... on PartitionRunConfig {
                                    yaml
                                }
                            }
                        }
                    }
                }
            }
        }
    }
"""

GET_PARTITION_SET_TAGS_QUERY = """
    query PartitionSetQuery($repositorySelector: RepositorySelector!, $partitionSetName: String!) {
        partitionSetOrError(repositorySelector: $repositorySelector, partitionSetName: $partitionSetName) {
            ...on PartitionSet {
                partitionsOrError(limit: 1) {
                    ... on Partitions {
                        results {
                            tagsOrError {
                                ... on PartitionTags {
                                    results {
                                        key
                                        value
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
"""

GET_PARTITION_SET_RUNS_QUERY = """
    query PartitionSetQuery($repositorySelector: RepositorySelector!, $partitionSetName: String!) {
        partitionSetOrError(repositorySelector: $repositorySelector, partitionSetName: $partitionSetName) {
            ...on PartitionSet {
                partitionsOrError {
                    ... on Partitions {
                        results {
                            name
                            runs {
                                runId
                            }
                        }
                    }
                }
            }
        }
    }
"""

GET_PARTITION_SET_STATUS_QUERY = """
    query PartitionSetQuery($repositorySelector: RepositorySelector!, $partitionSetName: String!) {
        partitionSetOrError(repositorySelector: $repositorySelector, partitionSetName: $partitionSetName) {
            ...on PartitionSet {
                id
                partitionStatusesOrError {
                    ... on PartitionStatuses {
                        results {
                            id
                            partitionName
                            runStatus
                        }
                    }
                }
            }
        }
    }
"""


class TestPartitionSets(ReadonlyGraphQLContextTestMatrix):
    def test_get_partition_sets_for_pipeline(self, graphql_context, snapshot):
        selector = infer_repository_selector(graphql_context)
        result = execute_dagster_graphql(
            graphql_context,
            GET_PARTITION_SETS_FOR_PIPELINE_QUERY,
            variables={"repositorySelector": selector, "pipelineName": "no_config_pipeline"},
        )

        assert result.data
        snapshot.assert_match(result.data)

        invalid_pipeline_result = execute_dagster_graphql(
            graphql_context,
            GET_PARTITION_SETS_FOR_PIPELINE_QUERY,
            variables={"repositorySelector": selector, "pipelineName": "invalid_pipeline"},
        )

        assert invalid_pipeline_result.data
        snapshot.assert_match(invalid_pipeline_result.data)

    def test_get_partition_set(self, graphql_context, snapshot):
        selector = infer_repository_selector(graphql_context)
        result = execute_dagster_graphql(
            graphql_context,
            GET_PARTITION_SET_QUERY,
            variables={"partitionSetName": "integer_partition", "repositorySelector": selector},
        )

        assert result.data
        snapshot.assert_match(result.data)

        invalid_partition_set_result = execute_dagster_graphql(
            graphql_context,
            GET_PARTITION_SET_QUERY,
            variables={"partitionSetName": "invalid_partition", "repositorySelector": selector},
        )

        assert (
            invalid_partition_set_result.data["partitionSetOrError"]["__typename"]
            == "PartitionSetNotFoundError"
        )
        assert invalid_partition_set_result.data

        snapshot.assert_match(invalid_partition_set_result.data)

    def test_get_partition_tags(self, graphql_context):
        selector = infer_repository_selector(graphql_context)
        result = execute_dagster_graphql(
            graphql_context,
            GET_PARTITION_SET_TAGS_QUERY,
            variables={"partitionSetName": "integer_partition", "repositorySelector": selector},
        )

        assert not result.errors
        assert result.data
        partitions = result.data["partitionSetOrError"]["partitionsOrError"]["results"]
        assert len(partitions) == 1
        sorted_items = sorted(partitions[0]["tagsOrError"]["results"], key=lambda item: item["key"])
        tags = OrderedDict({item["key"]: item["value"] for item in sorted_items})
        assert tags == {
            "foo": "0",
            "dagster/partition": "0",
            "dagster/partition_set": "integer_partition",
        }


class TestPartitionSetRuns(ExecutingGraphQLContextTestMatrix):
    def test_get_partition_runs(self, graphql_context):
        repository_selector = infer_repository_selector(graphql_context)
        result = execute_dagster_graphql_and_finish_runs(
            graphql_context,
            LAUNCH_PARTITION_BACKFILL_MUTATION,
            variables={
                "backfillParams": {
                    "selector": {
                        "repositorySelector": repository_selector,
                        "partitionSetName": "integer_partition",
                    },
                    "partitionNames": ["2", "3"],
                    "forceSynchronousSubmission": True,
                }
            },
        )
        assert not result.errors
        assert result.data["launchPartitionBackfill"]["__typename"] == "LaunchBackfillSuccess"
        assert len(result.data["launchPartitionBackfill"]["launchedRunIds"]) == 2
        run_ids = result.data["launchPartitionBackfill"]["launchedRunIds"]

        result = execute_dagster_graphql(
            graphql_context,
            query=GET_PARTITION_SET_RUNS_QUERY,
            variables={
                "partitionSetName": "integer_partition",
                "repositorySelector": repository_selector,
            },
        )
        assert not result.errors
        assert result.data
        partitions = result.data["partitionSetOrError"]["partitionsOrError"]["results"]
        assert len(partitions) == 10
        for partition in partitions:
            if partition["name"] not in ("2", "3"):
                assert len(partition["runs"]) == 0
            else:
                assert len(partition["runs"]) == 1
                assert partition["runs"][0]["runId"] in run_ids

    def test_get_partition_status(self, graphql_context):
        repository_selector = infer_repository_selector(graphql_context)
        result = execute_dagster_graphql_and_finish_runs(
            graphql_context,
            LAUNCH_PARTITION_BACKFILL_MUTATION,
            variables={
                "backfillParams": {
                    "selector": {
                        "repositorySelector": repository_selector,
                        "partitionSetName": "integer_partition",
                    },
                    "partitionNames": ["2", "3"],
                    "forceSynchronousSubmission": True,
                }
            },
        )
        assert not result.errors
        assert result.data["launchPartitionBackfill"]["__typename"] == "LaunchBackfillSuccess"
        assert len(result.data["launchPartitionBackfill"]["launchedRunIds"]) == 2

        result = execute_dagster_graphql(
            graphql_context,
            query=GET_PARTITION_SET_STATUS_QUERY,
            variables={
                "partitionSetName": "integer_partition",
                "repositorySelector": repository_selector,
            },
        )
        assert not result.errors
        assert result.data
        partitionStatuses = result.data["partitionSetOrError"]["partitionStatusesOrError"][
            "results"
        ]
        assert len(partitionStatuses) == 10
        for partitionStatus in partitionStatuses:
            if partitionStatus["partitionName"] in ("2", "3"):
                assert partitionStatus["runStatus"] == "SUCCESS"
            else:
                assert partitionStatus["runStatus"] is None

        result = execute_dagster_graphql_and_finish_runs(
            graphql_context,
            LAUNCH_PARTITION_BACKFILL_MUTATION,
            variables={
                "backfillParams": {
                    "selector": {
                        "repositorySelector": repository_selector,
                        "partitionSetName": "integer_partition",
                    },
                    "partitionNames": [str(num) for num in range(10)],
                    "forceSynchronousSubmission": True,
                }
            },
        )
        assert not result.errors
        assert result.data["launchPartitionBackfill"]["__typename"] == "LaunchBackfillSuccess"
        assert len(result.data["launchPartitionBackfill"]["launchedRunIds"]) == 10

        result = execute_dagster_graphql(
            graphql_context,
            query=GET_PARTITION_SET_STATUS_QUERY,
            variables={
                "partitionSetName": "integer_partition",
                "repositorySelector": repository_selector,
            },
        )
        assert not result.errors
        assert result.data
        partitionStatuses = result.data["partitionSetOrError"]["partitionStatusesOrError"][
            "results"
        ]
        assert len(partitionStatuses) == 10
        for partitionStatus in partitionStatuses:
            assert partitionStatus["runStatus"] == "SUCCESS"



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots['TestSensors.test_get_sensor[readonly_in_memory_instance_lazy_repository] 1'] = {
    '__typename': 'Sensor',
    'id': 'always_no_config_sensor:no_config_pipeline',
    'minIntervalSeconds': 30,
    'mode': 'default',
    'name': 'always_no_config_sensor',
    'nextTick': None,
    'pipelineName': 'no_config_pipeline',
    'sensorState': {
        'runs': [
        ],
        'runsCount': 0,
        'status': 'STOPPED',
        'ticks': [
        ]
    },
    'solidSelection': None
}

snapshots['TestSensors.test_get_sensor[readonly_in_memory_instance_managed_grpc_env] 1'] = {
    '__typename': 'Sensor',
    'id': 'always_no_config_sensor:no_config_pipeline',
    'minIntervalSeconds': 30,
    'mode': 'default',
    'name': 'always_no_config_sensor',
    'nextTick': None,
    'pipelineName': 'no_config_pipeline',
    'sensorState': {
        'runs': [
        ],
        'runsCount': 0,
        'status': 'STOPPED',
        'ticks': [
        ]
    },
    'solidSelection': None
}

snapshots['TestSensors.test_get_sensor[readonly_in_memory_instance_multi_location] 1'] = {
    '__typename': 'Sensor',
    'id': 'always_no_config_sensor:no_config_pipeline',
    'minIntervalSeconds': 30,
    'mode': 'default',
    'name': 'always_no_config_sensor',
    'nextTick': None,
    'pipelineName': 'no_config_pipeline',
    'sensorState': {
        'runs': [
        ],
        'runsCount': 0,
        'status': 'STOPPED',
        'ticks': [
        ]
    },
    'solidSelection': None
}

snapshots['TestSensors.test_get_sensor[readonly_sqlite_instance_deployed_grpc_env] 1'] = {
    '__typename': 'Sensor',
    'id': 'always_no_config_sensor:no_config_pipeline',
    'minIntervalSeconds': 30,
    'mode': 'default',
    'name': 'always_no_config_sensor',
    'nextTick': None,
    'pipelineName': 'no_config_pipeline',
    'sensorState': {
        'runs': [
        ],
        'runsCount': 0,
        'status': 'STOPPED',
        'ticks': [
        ]
    },
    'solidSelection': None
}

snapshots['TestSensors.test_get_sensor[readonly_sqlite_instance_lazy_repository] 1'] = {
    '__typename': 'Sensor',
    'id': 'always_no_config_sensor:no_config_pipeline',
    'minIntervalSeconds': 30,
    'mode': 'default',
    'name': 'always_no_config_sensor',
    'nextTick': None,
    'pipelineName': 'no_config_pipeline',
    'sensorState': {
        'runs': [
        ],
        'runsCount': 0,
        'status': 'STOPPED',
        'ticks': [
        ]
    },
    'solidSelection': None
}

snapshots['TestSensors.test_get_sensor[readonly_sqlite_instance_managed_grpc_env] 1'] = {
    '__typename': 'Sensor',
    'id': 'always_no_config_sensor:no_config_pipeline',
    'minIntervalSeconds': 30,
    'mode': 'default',
    'name': 'always_no_config_sensor',
    'nextTick': None,
    'pipelineName': 'no_config_pipeline',
    'sensorState': {
        'runs': [
        ],
        'runsCount': 0,
        'status': 'STOPPED',
        'ticks': [
        ]
    },
    'solidSelection': None
}

snapshots['TestSensors.test_get_sensor[readonly_sqlite_instance_multi_location] 1'] = {
    '__typename': 'Sensor',
    'id': 'always_no_config_sensor:no_config_pipeline',
    'minIntervalSeconds': 30,
    'mode': 'default',
    'name': 'always_no_config_sensor',
    'nextTick': None,
    'pipelineName': 'no_config_pipeline',
    'sensorState': {
        'runs': [
        ],
        'runsCount': 0,
        'status': 'STOPPED',
        'ticks': [
        ]
    },
    'solidSelection': None
}

snapshots['TestSensors.test_get_sensors[readonly_in_memory_instance_lazy_repository] 1'] = [
    {
        'description': None,
        'id': 'always_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'always_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'custom_interval_sensor:no_config_pipeline',
        'minIntervalSeconds': 60,
        'mode': 'default',
        'name': 'custom_interval_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'multi_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'multi_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'never_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'never_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'once_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'once_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    }
]

snapshots['TestSensors.test_get_sensors[readonly_in_memory_instance_managed_grpc_env] 1'] = [
    {
        'description': None,
        'id': 'always_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'always_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'custom_interval_sensor:no_config_pipeline',
        'minIntervalSeconds': 60,
        'mode': 'default',
        'name': 'custom_interval_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'multi_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'multi_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'never_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'never_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'once_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'once_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    }
]

snapshots['TestSensors.test_get_sensors[readonly_in_memory_instance_multi_location] 1'] = [
    {
        'description': None,
        'id': 'always_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'always_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'custom_interval_sensor:no_config_pipeline',
        'minIntervalSeconds': 60,
        'mode': 'default',
        'name': 'custom_interval_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'multi_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'multi_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'never_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'never_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'once_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'once_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    }
]

snapshots['TestSensors.test_get_sensors[readonly_sqlite_instance_deployed_grpc_env] 1'] = [
    {
        'description': None,
        'id': 'always_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'always_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'custom_interval_sensor:no_config_pipeline',
        'minIntervalSeconds': 60,
        'mode': 'default',
        'name': 'custom_interval_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'multi_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'multi_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'never_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'never_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'once_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'once_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    }
]

snapshots['TestSensors.test_get_sensors[readonly_sqlite_instance_lazy_repository] 1'] = [
    {
        'description': None,
        'id': 'always_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'always_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'custom_interval_sensor:no_config_pipeline',
        'minIntervalSeconds': 60,
        'mode': 'default',
        'name': 'custom_interval_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'multi_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'multi_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'never_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'never_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'once_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'once_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    }
]

snapshots['TestSensors.test_get_sensors[readonly_sqlite_instance_managed_grpc_env] 1'] = [
    {
        'description': None,
        'id': 'always_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'always_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'custom_interval_sensor:no_config_pipeline',
        'minIntervalSeconds': 60,
        'mode': 'default',
        'name': 'custom_interval_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'multi_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'multi_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'never_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'never_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'once_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'once_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    }
]

snapshots['TestSensors.test_get_sensors[readonly_sqlite_instance_multi_location] 1'] = [
    {
        'description': None,
        'id': 'always_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'always_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'custom_interval_sensor:no_config_pipeline',
        'minIntervalSeconds': 60,
        'mode': 'default',
        'name': 'custom_interval_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'multi_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'multi_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'never_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'never_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    },
    {
        'description': None,
        'id': 'once_no_config_sensor:no_config_pipeline',
        'minIntervalSeconds': 30,
        'mode': 'default',
        'name': 'once_no_config_sensor',
        'pipelineName': 'no_config_pipeline',
        'sensorState': {
            'runs': [
            ],
            'runsCount': 0,
            'status': 'STOPPED',
            'ticks': [
            ]
        },
        'solidSelection': None
    }
]



import graphene
from dagster import check
from dagster.core.host_representation import ExternalSchedule, ScheduleSelector
from dagster.core.host_representation.selector import RepositorySelector
from dagster.core.scheduler.job import JobTickStatsSnapshot

from ...implementation.fetch_schedules import (
    reconcile_scheduler_state,
    start_schedule,
    stop_schedule,
)
from ..errors import (
    GraphenePythonError,
    GrapheneRepositoryNotFoundError,
    GrapheneScheduleNotFoundError,
    GrapheneSchedulerNotDefinedError,
)
from ..inputs import GrapheneRepositorySelector, GrapheneScheduleSelector
from ..jobs import GrapheneJobState
from .schedules import (
    GrapheneSchedule,
    GrapheneScheduleOrError,
    GrapheneSchedules,
    GrapheneSchedulesOrError,
)
from .ticks import GrapheneJobTickStatus


class GrapheneScheduleStatus(graphene.Enum):
    RUNNING = "RUNNING"
    STOPPED = "STOPPED"
    ENDED = "ENDED"

    class Meta:
        name = "ScheduleStatus"


class GrapheneScheduler(graphene.ObjectType):
    scheduler_class = graphene.String()

    class Meta:
        name = "Scheduler"


class GrapheneSchedulerOrError(graphene.Union):
    class Meta:
        types = (GrapheneScheduler, GrapheneSchedulerNotDefinedError, GraphenePythonError)
        name = "SchedulerOrError"


class GrapheneScheduleTickStatsSnapshot(graphene.ObjectType):
    ticks_started = graphene.NonNull(graphene.Int)
    ticks_succeeded = graphene.NonNull(graphene.Int)
    ticks_skipped = graphene.NonNull(graphene.Int)
    ticks_failed = graphene.NonNull(graphene.Int)

    class Meta:
        name = "ScheduleTickStatsSnapshot"

    def __init__(self, stats):
        super().__init__(
            ticks_started=stats.ticks_started,
            ticks_succeeded=stats.ticks_succeeded,
            ticks_skipped=stats.ticks_skipped,
            ticks_failed=stats.ticks_failed,
        )
        self._stats = check.inst_param(stats, "stats", JobTickStatsSnapshot)


class GrapheneReconcileSchedulerStateSuccess(graphene.ObjectType):
    message = graphene.NonNull(graphene.String)

    class Meta:
        name = "ReconcileSchedulerStateSuccess"


class GrapheneReconcileSchedulerStateMutationResult(graphene.Union):
    class Meta:
        types = (GraphenePythonError, GrapheneReconcileSchedulerStateSuccess)
        name = "ReconcileSchedulerStateMutationResult"


class GrapheneReconcileSchedulerStateMutation(graphene.Mutation):
    Output = graphene.NonNull(GrapheneReconcileSchedulerStateMutationResult)

    class Arguments:
        repository_selector = graphene.NonNull(GrapheneRepositorySelector)

    class Meta:
        name = "ReconcileSchedulerStateMutation"

    def mutate(self, graphene_info, repository_selector):
        return reconcile_scheduler_state(
            graphene_info, RepositorySelector.from_graphql_input(repository_selector)
        )


class GrapheneScheduleStateResult(graphene.ObjectType):
    scheduleState = graphene.NonNull(GrapheneJobState)

    class Meta:
        name = "ScheduleStateResult"


class GrapheneScheduleMutationResult(graphene.Union):
    class Meta:
        types = (GraphenePythonError, GrapheneScheduleStateResult)
        name = "ScheduleMutationResult"


class GrapheneStartScheduleMutation(graphene.Mutation):
    Output = graphene.NonNull(GrapheneScheduleMutationResult)

    class Arguments:
        schedule_selector = graphene.NonNull(GrapheneScheduleSelector)

    class Meta:
        name = "StartScheduleMutation"

    def mutate(self, graphene_info, schedule_selector):
        return start_schedule(graphene_info, ScheduleSelector.from_graphql_input(schedule_selector))


class GrapheneStopRunningScheduleMutation(graphene.Mutation):
    Output = graphene.NonNull(GrapheneScheduleMutationResult)

    class Arguments:
        schedule_origin_id = graphene.NonNull(graphene.String)

    class Meta:
        name = "StopRunningScheduleMutation"

    def mutate(self, graphene_info, schedule_origin_id):
        return stop_schedule(graphene_info, schedule_origin_id)


def types():
    from .ticks import (
        GrapheneScheduleTick,
        GrapheneScheduleTickFailureData,
        GrapheneScheduleTickSpecificData,
        GrapheneScheduleTickSuccessData,
    )

    # Double check mutations don't appear twice
    return [
        GrapheneJobTickStatus,
        GrapheneReconcileSchedulerStateMutation,
        GrapheneReconcileSchedulerStateMutationResult,
        GrapheneReconcileSchedulerStateSuccess,
        GrapheneSchedule,
        GrapheneScheduleMutationResult,
        GrapheneScheduleOrError,
        GrapheneScheduler,
        GrapheneSchedulerOrError,
        GrapheneSchedules,
        GrapheneSchedulesOrError,
        GrapheneScheduleStateResult,
        GrapheneScheduleStatus,
        GrapheneScheduleTick,
        GrapheneScheduleTickFailureData,
        GrapheneScheduleTickSpecificData,
        GrapheneScheduleTickStatsSnapshot,
        GrapheneScheduleTickSuccessData,
        GrapheneStartScheduleMutation,
        GrapheneStopRunningScheduleMutation,
    ]
